<?php

namespace Drupal\searchstax\EventSubscriber;

use Drupal\Core\Logger\LoggerChannelInterface;
use Drupal\search_api_solr\Solarium\EventDispatcher\EventProxy;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Solarium\Core\Event\PreExecuteRequest;

/**
 * Changes the endpoint used for Solr search requests to SearchStudio.
 */
class SolariumSubscriber implements EventSubscriberInterface {

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The "searchstax" logger channel.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  public function __construct(ConfigFactoryInterface $config_factory, LoggerChannelInterface $logger) {
    $this->configFactory = $config_factory;
    $this->logger = $logger;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    return [
      PreExecuteRequest::class => 'handlePreExecuteRequestEvent',
    ];
  }

  /**
   * Reacts to the Solarium "pre-execute request" event.
   *
   * @param \Solarium\Core\Event\PreExecuteRequest|\Drupal\search_api_solr\Solarium\EventDispatcher\EventProxy $event
   *   The event.
   */
  public function handlePreExecuteRequestEvent($event) {
    if (!($event instanceof PreExecuteRequest || $event instanceof EventProxy)) {
      $vars = [
        '@method' => '\Drupal\searchstax\EventSubscriber\SolariumSubscriber::handlePreExecuteRequestEvent()',
        '@expected' => PreExecuteRequest::class,
        '@actual' => is_object($event) ? get_class($event) : gettype($event) . ' (not an object)',
      ];
      $this->logger->error('@method received object of unexpected class for argument 1. Expected: @expected; actual: @actual.', $vars);
      return;
    }

    // Re-route all "/select" requests to the "/emselect" SearchStudio handler.
    $request = $event->getRequest();
    $options = $request->getOptions();
    if (($options['handler'] ?? NULL) === 'select') {
      // Check whether this is even a SearchStax endpoint, and not a regular
      // Solr server.
      $host = $event->getEndpoint()->getHost();
      if (substr($host, -15) !== '.searchstax.com') {
        return;
      }

      // Check whether user enabled this behavior.
      $enabled = $this->configFactory->get('searchstax.settings')
        ->get('searches_via_searchstudio');
      if (!$enabled) {
        return;
      }

      // Check whether this is even

      $options['handler'] = 'emselect';
      $request->setOptions($options);
    }
  }

}
