<?php

namespace Drupal\searchstax\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Provides a form for changing the module's settings.
 */
class SettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'searchstax_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'searchstax.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('searchstax.settings');

    $form['analytics_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Analytics key'),
      '#description' => $this->t('The SearchStudio analytics key associated with this website.'),
      '#default_value' => $config->get('analytics_key'),
    ];
    // @todo Check whether /emselect is available before allowing to enable?
    $form['searches_via_searchstudio'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Re-route searches through SearchStudio'),
      '#description' => $this->t('Re-route all Solr searches through the SearchStudio search handler, to immediately apply adjustments you make in SearchStudio.'),
      '#default_value' => (bool) $config->get('searches_via_searchstudio'),
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config('searchstax.settings')
      ->set('analytics_key', $form_state->getValue('analytics_key'))
      ->set('searches_via_searchstudio', $form_state->getValue('searches_via_searchstudio'))
      ->save();

    parent::submitForm($form, $form_state);
  }

}
